#!/usr/bin/env python

"""***************************************************************************
**
** Copyright (C) 2005-2005 Trolltech AS. All rights reserved.
**
** This file is part of the example classes of the Qt Toolkit.
**
** This file may be used under the terms of the GNU General Public
** License version 2.0 as published by the Free Software Foundation
** and appearing in the file LICENSE.GPL included in the packaging of
** this file.  Please review the following information to ensure GNU
** General Public Licensing requirements will be met:
** http://www.trolltech.com/products/qt/opensource.html
**
** If you are unsure which license is appropriate for your use, please
** review the following information:
** http://www.trolltech.com/products/qt/licensing.html or contact the
** sales department at sales@trolltech.com.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
***************************************************************************"""

import sys
from PyQt4 import QtCore, QtGui

import editabletreemodel_rc
from ui_mainwindow import Ui_MainWindow

class TreeItem:
    def __init__(self, data, parent=None):
        self.parentItem = parent
        self.itemData = data
        self.childItems = []

    def child(self, row):
        return self.childItems[row]

    def childCount(self):
        return len(self.childItems)

    def columnCount(self):
        return len(self.itemData)

    def data(self, column):
        return self.itemData[column]

    def insertChildren(self, position, count, columns):
        if position < 0 or position > len(self.childItems):
            return False
        for row in range(count):
            data = []
            for v in range(columns):
                data.append(QtCore.QVariant("[No data]"))
            item = TreeItem(data, self)
            self.childItems.insert(position, item)
        return True

    def insertColumns(self, position, columns):
        if position < 0 or position > len(self.itemData):
            return False
        for column in range(columns):
            self.itemData.insert(position, QtCore.QVariant("[No data]"))
        for child in self.childItems:
            child.insertColumns(position, columns)
        return True

    def parent(self):
        return self.parentItem

    def removeChildren(self, position, count):
        if position < 0 or position + count > len(self.childItems):
            return False
        for row in range(count):
            self.childItems.pop(position)
        return True

    def removeColumns(self, position, columns):
        if position < 0 or position + columns > len(self.itemData):
            return False
        for column in range(columns):
            self.itemData.pop(position)
        for child in self.childItems:
            child.removeColumns(position, columns)
        return True

    def childNumber(self):
        if self.parentItem != None:
            return self.parentItem.childItems.index(self)
        return 0

    def setData(self, column, value):
        if column < 0 or column >= len(self.itemData):
            return False
        if type(value) == type(QtCore.QVariant()):
            self.itemData[column] = value.toString()
        else:
            self.itemData[column] = value
        return True

class TreeModel(QtCore.QAbstractItemModel):
    def __init__(self, headers, data, parent=None):
        QtCore.QAbstractItemModel.__init__(self, parent)

        rootData = []
        for header in headers:
            rootData.append(header)
        self.rootItem = TreeItem(rootData)
        self.setupModelData(data.split("\n"), self.rootItem)

    def data(self, index, role):
        if not index.isValid():
            return QtCore.QVariant()

        if role != QtCore.Qt.DisplayRole and role != QtCore.Qt.EditRole:
            return QtCore.QVariant()

        item = self.getItem(index)
        return QtCore.QVariant(item.data(index.column()))

    def headerData(self, section, orientation, role=QtCore.Qt.DisplayRole):
        if orientation == QtCore.Qt.Horizontal and role == QtCore.Qt.DisplayRole:
            return QtCore.QVariant(self.rootItem.data(section))

        return QtCore.QVariant()

    def index(self, row, column, parent=QtCore.QModelIndex()):
        if parent.isValid() and parent.column() != 0:
            return QtCore.QModelIndex()

        parentItem = self.getItem(parent)
        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        else:
            return QtCore.QModelIndex()

    def parent(self, index):
        if not index.isValid():
            return QtCore.QModelIndex()

        childItem = self.getItem(index)
        parentItem = childItem.parent()

        if parentItem == self.rootItem:
            return QtCore.QModelIndex()

        return self.createIndex(parentItem.childNumber(), 0, parentItem)

    def rowCount(self, parent=QtCore.QModelIndex()):
        parentItem = self.getItem(parent)
        return parentItem.childCount()

    def columnCount(self, parent=QtCore.QModelIndex()):
        return self.rootItem.columnCount()

    def flags(self, index):
        if not index.isValid():
            return 0

        return QtCore.Qt.ItemIsEnabled | QtCore.Qt.ItemIsEditable | QtCore.Qt.ItemIsSelectable

    def setData(self, index, value, role=QtCore.Qt.EditRole):
        if role != QtCore.Qt.EditRole:
            return False

        item = self.getItem(index)
        result = item.setData(index.column(), value)
        if result:
            self.emit(QtCore.SIGNAL("dataChanged(const QModelIndex &, const QModelIndex &)"), index, index)

        return result

    def setHeaderData(self, section, orientation, value, role=QtCore.Qt.EditRole):
        if role != QtCore.Qt.EditRole or orientation != QtCore.Qt.Horizontal:
            return False

        result = self.rootItem.setData(section, value)
        if result:
            self.emit(QtCore.SIGNAL("headerDataChanged(Qt::orientation, int, int)"), orientation, section, section)

        return result

    def insertColumns(self, position, columns, parent=QtCore.QModelIndex()):
        self.beginInsertColumns(parent, position, position + columns - 1)
        success = self.rootItem.insertColumns(position, columns)
        self.endInsertColumns()
        return success

    def removeColumns(self, position, columns, parent=QtCore.QModelIndex()):
        self.beginRemoveColumns(parent, position, position + columns - 1)
        success = self.rootItem.removeColumns(position, columns)
        self.endRemoveColumns()
        if self.rootItem.columnCount() == 0:
            self.removeRows(0, rowCount())
        return success

    def insertRows(self, position, rows, parent=QtCore.QModelIndex()):
        parentItem = self.getItem(parent)
        self.beginInsertRows(parent, position, position + rows - 1)
        success = parentItem.insertChildren(position, rows, self.rootItem.columnCount())
        self.endInsertRows()
        return success

    def removeRows(self, position, rows, parent=QtCore.QModelIndex()):
        parentItem = self.getItem(parent)
        self.beginRemoveRows(parent, position, position + rows - 1)
        success = parentItem.removeChildren(position, rows)
        self.endRemoveRows()
        return success

    def setupModelData(self, lines, parent):
        parents = []
        indentations = []
        parents.append(parent)
        indentations.append(0)

        number = 0

        while number < len(lines):
            position = 0
            while position < len(lines[number]):
                if lines[number][position] != " ":
                    break
                position += 1

            lineData = lines[number][position:].trimmed()

            if not lineData.isEmpty():
                # Read the column data from the rest of the line.
                columnStrings = lineData.split("\t", QtCore.QString.SkipEmptyParts)
                columnData = []
                for column in range(0, len(columnStrings)):
                    columnData.append(columnStrings[column])

                if position > indentations[-1]:
                    # The last child of the current parent is now the new parent
                    # unless the current parent has no children.

                    if parents[-1].childCount() > 0:
                        parents.append(parents[-1].child(parents[-1].childCount() - 1))
                        indentations.append(position)

                else:
                    while position < indentations[-1] and len(parents) > 0:
                        parents.pop()
                        indentations.pop()

                # Append a new item to the current parent's list of children.
                parent = parents[-1]
                parent.insertChildren(parent.childCount(), 1, self.rootItem.columnCount())
                for column in range(0, len(columnData)):
                    parent.child(parent.childCount() -1).setData(column, columnData[column])

            number += 1

    def getItem(self, index):
        if index.isValid():
            item = index.internalPointer()
            return item
        return self.rootItem


class MainWindow(QtGui.QMainWindow, Ui_MainWindow):
    def __init__(self, parent=None):
        QtGui.QMainWindow.__init__(self, parent)

        self.setupUi(self)

        headers = QtCore.QStringList()
        headers << self.tr("Title") << self.tr("Description")

        file = QtCore.QFile(":/default.txt")
        file.open(QtCore.QIODevice.ReadOnly)
        self.model = TreeModel(headers, QtCore.QString(file.readAll()))
        file.close()

        self.view.setModel(self.model)
        for column in range(self.model.columnCount(QtCore.QModelIndex())):
            self.view.resizeColumnToContents(column)

        self.connect(self.exitAction, QtCore.SIGNAL("triggered()"), 
            QtGui.qApp, QtCore.SLOT("quit()"))

        self.connect(self.view.selectionModel(),
            QtCore.SIGNAL("selectionChanged(const QItemSelection &, const QItemSelection &)"), self.updateActions)

        self.connect(self.actionsMenu, QtCore.SIGNAL("aboutToShow()"), self.updateActions)
        self.connect(self.insertRowAction, QtCore.SIGNAL("triggered()"), self.insertRow)
        self.connect(self.insertColumnAction, QtCore.SIGNAL("triggered()"), self.insertColumn)
        self.connect(self.removeRowAction, QtCore.SIGNAL("triggered()"), self.removeRow)
        self.connect(self.removeColumnAction, QtCore.SIGNAL("triggered()"), self.removeColumn)
        self.connect(self.insertChildAction, QtCore.SIGNAL("triggered()"), self.insertChild)

        self.updateActions()

    def insertChild(self):
        index = QtCore.QModelIndex(self.view.selectionModel().currentIndex())

        if self.model.columnCount(index) == 0:
            if not self.model.insertColumn(0, index):
                return

        if not self.model.insertRow(0, index):
            return

        for column in range(self.model.columnCount(index)):
            child = QtCore.QModelIndex(self.model.index(0, column, index))
            self.model.setData(child, QtCore.QVariant("[No data]"), QtCore.Qt.EditRole)
            if not self.model.headerData(column, QtCore.Qt.Horizontal).isValid():
                self.model.setHeaderData(column, QtCore.Qt.Horizontal, 
                        QtCore.QVariant("[No header]"), QtCore.Qt.EditRole)

        self.view.selectionModel().setCurrentIndex(self.model.index(0, 0, index),
            QtGui.QItemSelectionModel.ClearAndSelect)
        self.updateActions()

    def insertColumn(self, parent=QtCore.QModelIndex()):
        column = self.view.selectionModel().currentIndex().column()

        # Insert a column in the parent item.
        changed = self.model.insertColumn(column + 1, parent)
        if (changed):
            self.model.setHeaderData(column + 1, QtCore.Qt.Horizontal, 
                    QtCore.QVariant("[No header]"), QtCore.Qt.EditRole)

        self.updateActions()

        return changed

    def insertRow(self):
        index = QtCore.QModelIndex(self.view.selectionModel().currentIndex())

        if not self.model.insertRow(index.row()+1, index.parent()):
            return

        self.updateActions()

        for column in range(self.model.columnCount(index.parent())):
            child = QtCore.QModelIndex(self.model.index(index.row()+1, column, index.parent()))
            self.model.setData(child, QtCore.QVariant("[No data]"), QtCore.Qt.EditRole)

    def removeColumn(self, parent=QtCore.QModelIndex()):
        column = self.view.selectionModel().currentIndex().column()

        # Insert columns in each child of the parent item.
        changed = self.model.removeColumn(column, parent)

        if not parent.isValid() and changed:
            self.updateActions()

        return changed

    def removeRow(self):
        index = QtCore.QModelIndex(self.view.selectionModel().currentIndex())
        if (self.model.removeRow(index.row(), index.parent())):
            self.updateActions()

    def updateActions(self):
        hasSelection = not self.view.selectionModel().selection().isEmpty()
        self.removeRowAction.setEnabled(hasSelection)
        self.removeColumnAction.setEnabled(hasSelection)

        hasCurrent = self.view.selectionModel().currentIndex().isValid()
        self.insertRowAction.setEnabled(hasCurrent)
        self.insertColumnAction.setEnabled(hasCurrent)

        if (hasCurrent):
            self.view.closePersistentEditor(self.view.selectionModel().currentIndex())

            row = self.view.selectionModel().currentIndex().row()
            column = self.view.selectionModel().currentIndex().column()
            if (self.view.selectionModel().currentIndex().parent().isValid()):
                self.statusBar().showMessage(self.tr("Position: (%1,%2)").arg(row).arg(column))
            else:
                self.statusBar().showMessage(self.tr("Position: (%1,%2) in top level").arg(row).arg(column))


if __name__ == "__main__":
    app = QtGui.QApplication(sys.argv)
    window = MainWindow()
    window.show()
    sys.exit(app.exec_())
